
#include "CDispatchManager.h"
#include "CInputManager.h"
#include "CGameManager.h"
#include "CAIPlayer.h"
#include "CMenuSystem.h"
#include "CCameraManager.h"
#include "CFollowCamera.h"
#include "CSteeringBehavior.h"
#include "Constants.h"
#include "CShaderManager.h"
#include "CCollisionManager.h"
#include "CSceneManager.h"
#include "CEffectsManager.h"
#include "CShieldEffect.h"
#include "CGameTime.h"


CAIPlayer::CAIPlayer()
{
	//m_nWaypoint = 0;
	m_fCurrentSpeed = 0.0f;
	m_AIID = 0;

	m_fJumpAccumulator = 0.0f;
	m_fTurnAccumulator = 0.0f;
	/*m_fTurnIncrement = (D3DX_PI * 0.25f);
	m_fTurnMax = (float)(D3DX_PI * 0.5f);*/
	m_fTurnIncrement = 0.2f;
	m_fTurnMax = 0.7f;

	m_v3Velocity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	m_fIntendedSpeed = 0.0f;
	m_bIsPlayer = false;
	m_bRubberBand = false;
	m_bInput = false;
	m_bTurning = false;
	m_bAI = true;
	m_bPathFollow = true;
	m_bPositionUpdate = true;

	m_bSteerLeft = false;
	m_bSteerRight = false;
	m_bSpeedUp = false;
	m_bSpeedDown = false;
	m_bDisplayWhiskers = false;
	m_bAIActive = false;
	m_bBumping = false;

	m_fAlignmentRange = 0.00f;
	m_fAlignmentToWayPoint = 0.0f;

	m_bAlignLeft = false;
	m_bAlignRight = false;

	m_fCurrentBoost = 1.0f;
	m_fCurrentTick = 0.0f;
	//m_fMaxSpeed = 2.0f;
	m_nCurrentLap = 1;

	m_bOnRail = false;

	m_nLeftExhaustID = -1;
	m_nRightExhaustID = -1;

	m_nBoost = 3;
	m_fBoostFactor = 1.0f;
	m_fUseBoostTimer = 0.0f;
	m_nCurrentRing = 0;

	m_nWaypoint = 0;

	m_fTimeFromStart = 0.0f;
	m_bUpdateStartTime = false;
	m_bStartTimeUp = false;
	m_bUpdateShieldTimer = false;
	m_fCollisionTimer = 0.0f;
	m_fBumpTimer = 0.0f;
	m_fUseBoostTimer = 0.0f;
	m_bAllowCatchUpSpeed = false;

	m_fTimeToSpeedUp = 0.0f;
	m_fCatchUpDistance = 0.0f;
	m_fCatchUpSpeedMultiplier = 0.0f;
	m_fUseBoostDistance = 0.0f;
	m_fCatchUpOffDistance = 0.0f;

	m_bBoosting = false;
	m_bRailBoosting = false;
	m_fHandling = 1.5f;

	m_fFireDistance = 0.0f;
	m_fFireWeaponTimer = 0.0f;
	m_fWeaponRechargeTime = 0.0f;
	m_nFollowWaypoint = 0;
}

CAIPlayer::~CAIPlayer()
{
}

void CAIPlayer::Init()
{
	CRacer::Init();

	LoadAIValues();

	InitMe();



	DispatchManager().AddToBucket(UPDATE_OBJECT_BUCKET, (CBucketItem *)this);			
	DispatchManager().AddToBucket(UPDATE_COLLISION_BUCKET, (CBucketItem *)this);
	DispatchManager().AddToBucket(RENDER_PLAYER1_FRONTBACK_BUCKET, (CBucketItem *)this);


	/*m_hMatWorldViewProj	= ShaderManager().Shader()->GetParameterByName(0,"matWorldViewProj");
	m_hMatWorldIT		= ShaderManager().Shader()->GetParameterByName(0,"matWorldIT");
	m_hMatWorld			= ShaderManager().Shader()->GetParameterByName(0,"matWorld");
	m_hViewPosition		= ShaderManager().Shader()->GetParameterByName(0,"viewPosition");
	m_hTexTexture		= ShaderManager().Shader()->GetParameterByName(0,"texTexture");*/
}

void CAIPlayer::LoadAIValues()
{
	//CScript* aiScript = ScriptManager().Script("AI Values");
	//char buffer[128] ="";

	//sprintf_s(buffer, sizeof(buffer), "TIME_TO_SPEED_UP");
	//m_fTimeToSpeedUp = aiScript->ValueAsFloat(buffer);

	//sprintf_s(buffer, sizeof(buffer), "CATCH_UP_DISTANCE");
	//m_fCatchUpDistance = aiScript->ValueAsFloat(buffer);

	//sprintf_s(buffer, sizeof(buffer), "CATCH_UP_OFF_DISTANCE");
	//m_fCatchUpOffDistance = aiScript->ValueAsFloat(buffer);

	//sprintf_s(buffer, sizeof(buffer), "CATCH_UP_SPEED_MULTIPLIER");
	//m_fCatchUpSpeedMultiplier = aiScript->ValueAsFloat(buffer);

	//sprintf_s(buffer, sizeof(buffer), "USE_BOOST_DISTANCE");
	//m_fUseBoostDistance = aiScript->ValueAsFloat(buffer);
}

void CAIPlayer::InitMe()
{
	//	m_nModelID = AssetManager().GetAssetID("BS_LightShipModel.x");
	//	m_nModelID = AssetManager().GetAssetID("LandShark.x");
	//	m_nModelID = AssetManager().GetAssetID("BS_MediumShipE.x");
	//	m_nModelID = AssetManager().GetAssetID("BS_LightShip2C.x");
	m_nModelID = AssetManager().GetAssetID("BS_LightShip3D.x");

	m_nCurrentRing = 0;

	m_vWhiskers.resize(MAX_WHISKERS);
	m_bFinished = false;

	m_bvSphere.SetRadius(2.0f);

	m_fCurrentSpeed = 0.0f;

	m_nShipIntegrity = 100;
	m_nShieldLevel = 100;
	m_bAlive = true;
	m_nBoost = 3;

	tCourseLine &currCL = GameManager().Track().CourseLines()[m_nWaypoint];

	D3DXCreateCylinder(Direct3D().GetDirect3DDevice(), 0.05f, 0.05f, 1.0f, 16, 32, &m_pMeshVector, NULL);

	m_mTransform = currCL.mTransform;	
	m_mTransform.axis_pos += m_mTransform.axis_y * 2.0f;
	m_mTransform.axis_pos += m_mTransform.axis_z * (currCL.fLength * 0.5f);

	
	char buffer[128] ="";

	sprintf_s(buffer, sizeof(buffer), "AI_Follow_Camera%i", m_AIID);

	m_Camera = new CFollowCamera(buffer, D3DXVECTOR3(0.0f, 0.0f, 0.0f), this);
	m_iCameraIndex = CameraManager().LoadCamera(m_Camera);
	m_szCameraName = buffer;
	//m_HUD.Initialize();
	//m_HUD.SetInfo(1, 1.0f, 1.0f, 0, 0, 0.0f);
	//m_fMaxSpeed = 2.0f;
	m_nCurrentLap = 1;

	m_bAdjustToWaypoint = false;
	m_fTimeOnWaypoint = 0.0f;
	m_fTimeOnWaypointSinceLastAlign = 0.0f;

	//m_nLeftExhaustID = EffectsManager().CreateEffect(EFF_EXHAUST);
	//EffectsManager().GetEffect(m_nLeftExhaustID)->SetTarget(this);
	//EffectsManager().GetEffect(m_nLeftExhaustID)->SetPosition(D3DXVECTOR3(-0.95f, 0.0f, 2.5f));

	//m_nRightExhaustID = EffectsManager().CreateEffect(EFF_EXHAUST);
	//EffectsManager().GetEffect(m_nRightExhaustID)->SetTarget(this);
	//EffectsManager().GetEffect(m_nRightExhaustID)->SetPosition(D3DXVECTOR3(0.95f, 0.0f, 2.5f));
}

void CAIPlayer::Update(BUCKET_ID nParentID)
{
	if (!m_bAlive)
	{
		return;
	}

	switch(nParentID)
	{
	case UPDATE_OBJECT_BUCKET:
		UpdateMe();
		break;

	case RENDER_PLAYER1_FRONTBACK_BUCKET:
		RenderMe();
		break;	
	};	
}
bool CAIPlayer::ProcessCollision(CGameObject*	_OBJECT)
{
	float Distance;
	D3DXVECTOR3 toShip;
	D3DXVECTOR3 shipHeading;
	D3DXVECTOR3 targetPos;
	float side;
	int Trail;

	switch(_OBJECT->GetType())
	{
		///////////////////////////////////////////////////////////////
		///Collide with Ship/Wall, continue if Weapon or Powerup
		////////////////////////////////////////////////////////////////
		case OBJECT_SHIP:
			//TODO SHIP TO SHIP
			if (m_fBumpTimer > 1.0f)
			{
				m_fBumpTimer = 0.0f;
				toShip = m_mTransform.axis_pos - _OBJECT->Pos();
				D3DXVec3Normalize(&toShip, &toShip);
				side = D3DXVec3Dot(&toShip, &m_mTransform.axis_x);
				Distance = D3DXVec3Length(&toShip);

				if (side > 0)
				{
					// ON RIGHT
					m_mTransform.axis_pos += _OBJECT->Transform().axis_x * (this->m_bvSphere.GetRadius());
					TurnRight(1.5f);
				}
				else if (side < 0)
				{
					// ON LEFT
					m_mTransform.axis_pos += -_OBJECT->Transform().axis_x * (this->m_bvSphere.GetRadius());
					TurnLeft(1.5f);
				}

				TakeDamage(30);

				if (m_nShieldLevel > 0.0f)
				{
					Trail = EffectsManager().CreateEffect(EFF_SHIELD);
					if(Trail > -1)
					{
						EffectsManager().GetEffect(Trail)->SetTarget(this);
						dynamic_cast<CShieldEffect*>(EffectsManager().GetEffect(Trail))->SetShieldDirection(_OBJECT->Transform().axis_pos);
					}
				}
			}
			return true;
			break;
		case OBJECT_WEAPON:
			//TODO SHIP TO WEAPON
			TakeDamage(40);
			m_fCurrentSpeed -= 0.5f;

			if (m_nShieldLevel > 0.0f)
			{
				Trail = EffectsManager().CreateEffect(EFF_SHIELD);
				if(Trail > -1)
				{
					EffectsManager().GetEffect(Trail)->SetTarget(this);
					dynamic_cast<CShieldEffect*>(EffectsManager().GetEffect(Trail))->SetShieldDirection(_OBJECT->Transform().axis_pos);
				}
			}
			return false;
			break;
		case OBJECT_POWERUP:
			//TODO GIVE AI SECONDARY WEAPON
			return false;
			break;
		case OBJECT_WALL:
			//MAKE THEM DO THE DANCE!
			if (m_fCollisionTimer > 1.0f)
			{
				m_fCollisionTimer = 0.0f;
				TakeDamage(30);
			}
			return false;
			break;
		default:
			return false;
			break;
	}
}
void CAIPlayer::UpdatePathFollow()
{
	/////////////////////
	// Update Waypoint //
	/////////////////////

	int nOldWaypoint = m_nWaypoint;
	int nNextWaypoint = m_nWaypoint + 1;

	int nNumWaypoints = (int)GameManager().Track().CourseLines().size();

	////////////////////////////////////
	// check distance to the waypoint //
	////////////////////////////////////
	float result = 0.0f;

	do
	{
		nNextWaypoint = m_nWaypoint + 1;

		if (nNextWaypoint == nNumWaypoints)
			nNextWaypoint = 0;		

		D3DXVECTOR3 toShip(m_mTransform.axis_pos - GameManager().Track().CourseLines()[nNextWaypoint].mTransform.axis_pos);

		result = D3DXVec3Dot(&(-GameManager().Track().CourseLines()[nNextWaypoint].mTransform.axis_z), &toShip);

		if (result < 0.0f)
		{
			m_nWaypoint++;

			// We've completed 1 lap.
			if (m_nWaypoint == nNumWaypoints)
			{
				m_nWaypoint = 0;
				m_nCurrentLap++;
				m_nCurrentScore += 1000;

			} // end if

		} // end if

	} while (result < 0.0f);

	////if(m_bautopilot)
	////{
		int nFollowWaypoint  = m_nWaypoint + m_nFollowWaypoint;

		if (nFollowWaypoint >= nNumWaypoints)
			nFollowWaypoint -= nNumWaypoints;

		D3DXVECTOR3 toWaypoint(GameManager().Track().CourseLines()[nFollowWaypoint].mTransform.axis_pos - m_mTransform.axis_pos);
		D3DXVec3Normalize(&toWaypoint, &toWaypoint);

		// save original transform
		m_mRenderTransform.matrix_dx = m_mTransform.matrix_dx;

		result = D3DXVec3Dot(&m_mTransform.axis_x, &toWaypoint);

		// change driving left-right transform
		m_mTransform.RotateY(result * m_fHandling * m_fTick);			

		// change display left-right transform
		m_mRenderTransform.RotateY(result * 10.0f * m_fTick);

		result = D3DXVec3Dot(&m_mTransform.axis_y, &toWaypoint);

		// change driving up-down transform
		m_mTransform.RotateX(-result * m_fHandling * m_fTick);

		// change display up-down transform
		m_mRenderTransform.RotateX(-result * 10.0f * m_fTick);

		// swing ship around
		m_mRenderTransform.RotateY(D3DX_PI);

		m_fTurnAccumulator = 0.0f;
}

void CAIPlayer::UpdatePosition(float fTick)
{
	tCourseLine &currCL = GameManager().Track().CourseLines()[m_nWaypoint];

	///////////////////////////
	// Update position based //
	// on current direction  //
	///////////////////////////
	m_mTransform.axis_pos += m_mTransform.axis_z * m_fCurrentSpeed * currCL.fSpeed * fTick;
}

void CAIPlayer::UpdateInput(float fTick)
{
	//if(InputManager().Keyboard().GetKey(DIK_6))
	//{
	//	m_bAIActive = true;
	//}
	//if(InputManager().Keyboard().GetKey(DIK_7))
	//{
	//	m_bAIActive = false;
	//	m_tCurrentStats.m_fSpeed = 0.08f;
	//}

	//if(InputManager().Keyboard().GetKey(DIK_8))
	//{
	//	m_tCurrentStats.m_fSpeed -= 0.05f * fTick;
	//}
	//if(InputManager().Keyboard().GetKey(DIK_9))
	//{
	//	m_tCurrentStats.m_fSpeed += 0.05f * fTick;
	//}

	if(InputManager().Keyboard().GetKey(DIK_I))
	{
		m_fCurrentSpeed += 0.1f * fTick;
	}
	if(InputManager().Keyboard().GetKey(DIK_K))
	{
		m_fCurrentSpeed -= 0.1f * fTick;
	}

	/*if()
	{
	TurnLeft(fTick);
	}
	if()
	{
	TurnRight(fTick);
	}*/

	if (InputManager().Keyboard().GetKey(DIK_J))
		TurnLeft(1.0f * fTick);
	else if (InputManager().Keyboard().GetKey(DIK_L))
		TurnRight(1.0f * fTick);
	//else
	//{
	//	if (m_fTurnAccumulator < 0.0f)
	//	{
	//		TurnRight(1.5f * fTick);

	//		if (m_fTurnAccumulator > 0.0f)
	//			m_fTurnAccumulator = 0.0f;

	//	} // end if
	//	else if (m_fTurnAccumulator > 0.0f)
	//	{
	//		TurnLeft(1.5f * fTick);

	//		if (m_fTurnAccumulator < 0.0f)
	//			m_fTurnAccumulator = 0.0f;

	//	} // end else if

	//}
}

void CAIPlayer::UpdateBoundingVolume()
{
	////////////////////////////////////////
	// Set out bounding volume to our position
	////////////////////////////////////////
	m_bvSphere.SetCenter(m_mTransform.axis_pos);
}

void CAIPlayer::UpdateMe()
{
	CRacer::Update();

	//float fTick = GameManager().Timer().GetDeltaTime();
	float fTick = (float)GameTime().GameTimeStep();
	m_fCurrentTick = fTick;
	m_fTick = fTick;

	if(!this->m_bFinished)
	{
		this->m_fFinishTime += m_fTick;
		this->m_nFinishRank = m_nRacePosition;
	}

	if (m_nShipIntegrity <= 0)
	{
		this->m_bFinished = true;
		m_nFinishRank = m_nRacePosition;
		this->m_fFinishTime = -1.0f;
		m_bAlive = false;

		if(m_nGlowID != -1)
			EffectsManager().GetEffect(m_nGlowID)->Shutdown();

		if(m_nLeftExhaustID != -1)
			EffectsManager().GetEffect(m_nLeftExhaustID)->Shutdown();

		if(m_nRightExhaustID != -1)
			EffectsManager().GetEffect(m_nRightExhaustID)->Shutdown();

		DispatchManager().RemoveFromBucket(UPDATE_OBJECT_BUCKET, (CBucketItem *)this);			
		DispatchManager().RemoveFromBucket(UPDATE_COLLISION_BUCKET, (CBucketItem *)this);
		DispatchManager().RemoveFromBucket(RENDER_PLAYER1_FRONTBACK_BUCKET, (CBucketItem *)this);
	}

	if (!m_bAlive)
	{
		return;
	}

	m_v3OldPosition = m_mTransform.axis_pos;

	if (m_bPathFollow)
	{
		UpdatePathFollow();
		//AvoidWalls();
	}

	tCourseLine &currCL = GameManager().Track().CourseLines()[m_nWaypoint];

	/*if ( (m_nWaypoint > 75 && m_nWaypoint < 84) || (m_nWaypoint > 154 && m_nWaypoint < 168)  )
		m_bOnRail = true;
	else
		m_bOnRail = false;*/

	m_bSteerLeft = false;
	m_bSteerRight = false;

	// Check if we want to update the start time
	// The ai players initially start out going at set speeds
	// to spread them out.  After the timeToSpeedUp is reached,
	// ai players will now attempt to go at fullspeed throughout the race.

	float timeToSpeedUp = 30;
	if (m_bUpdateStartTime)
	{
		m_fTimeFromStart += fTick;

		if (m_fTimeFromStart >= m_fTimeToSpeedUp)
		{
			m_bUpdateStartTime = false;
			m_bStartTimeUp = true;
			AccelerateTo(m_fMaxSpeed);
		}
	}

	m_fCollisionTimer += fTick;
	m_fBumpTimer += fTick;
	m_fUseBoostTimer += fTick;
	m_fTimeOnWaypoint += fTick;
	m_fTimeOnWaypointSinceLastAlign += fTick;
	m_fFireWeaponTimer += fTick;

	// Recharge the shield when the timer is up
	if (m_bUpdateShieldTimer)
	{
		m_fShieldRechargeTimer += fTick;

		if (m_fShieldRechargeTimer >= m_fShieldRecharge)
		{
			m_bUpdateShieldTimer = false;
			m_nShieldLevel = 100;
			m_fShieldRechargeTimer = 0.0f;
		}
	}
	
	// Once the start time is up, we want to make sure the ai
	// always attempts to go at full speed, even if the player
	// does something that makes them slow down.
	if (m_bStartTimeUp && !m_bOnRail)
	{
		if (m_fCurrentSpeed < m_fMaxSpeed)
		{
			AccelerateTo(m_fMaxSpeed);
		}

		if (m_fCurrentSpeed > m_fMaxSpeed && !m_bAllowCatchUpSpeed)
		{
			m_fCurrentSpeed = m_fMaxSpeed;
		}

		// If the ai player is very far away from the player
		// allow them to go more than their max speed to catch up
		if (DistanceFromPlayer() < m_fCatchUpDistance && !m_bAllowCatchUpSpeed && m_nBoost == 0)
		{
			m_bAllowCatchUpSpeed = true;
			AccelerateTo(m_fMaxSpeed * m_fCatchUpSpeedMultiplier);
		}

		if (DistanceFromPlayer() < m_fUseBoostDistance && m_nBoost > 0 && m_fUseBoostTimer > 5.0f)
		{
			m_fUseBoostTimer = 0.0f;
			Boost(BoostFactor());
		}

		if (DistanceFromPlayer() > m_fCatchUpOffDistance)
		{
			m_bAllowCatchUpSpeed = false;
			AccelerateTo(m_fMaxSpeed);
		}

		vector<CRacer*> racers = GameManager().GetAllRacers();

		for (int i = 0; i < racers.size(); i++)
		{
			float Distance;
			D3DXVECTOR3 toShip;
			D3DXVECTOR3 shipHeading;
			D3DXVECTOR3 targetPos;
			float side;

			toShip = racers[i]->Pos() - m_mTransform.axis_pos;
			D3DXVec3Normalize(&toShip, &toShip);
			side = D3DXVec3Dot(&toShip, &-m_mTransform.axis_z);
			Distance = D3DXVec3Length(&toShip);

			if (racers[i]->IsPlayer())
			{
				bool adjustFromRacer = false;

				if (DistanceFromPlayer() > 0 && DistanceFromPlayer() < 3)
				{
					adjustFromRacer = true;
				}

				if (side > 0.0f)
				{
					//TargetShip is in back of the player - CORRECT
					if (adjustFromRacer)
					{
						m_fCurrentSpeed -= 0.2f;
					}
				}
				if (side < 0.0f)
				{
					//Target ship is in front of player
					if (adjustFromRacer)
					{
						m_fCurrentSpeed += 0.2f;
					}

					if (DistanceFromPlayer() < m_fFireDistance && DistanceFromPlayer() > -200.0f && m_fFireWeaponTimer > m_fWeaponRechargeTime && !m_bBoosting && !m_bOnRail)
					{
						if (m_eSecondaryWeapon > 0)
						{
							m_fFireWeaponTimer = 0.0f;
							FireBullet(m_eSecondaryWeapon);
							m_eSecondaryWeapon = WEAPON_LASER;
						}
						else
						{
							m_fFireWeaponTimer = 0.0f;
							FireBullet(WEAPON_LASER);
						}
					}
				}
			}
			else
			{
				bool adjustFromRacer = false;
				if(DistanceFrom(racers[i]) > 0 &&  DistanceFrom(racers[i]) < 3)
				{
					adjustFromRacer = true;
				}

				if (side > 0.0f)
				{
					//TargetShip is in back of the player - CORRECT
					if (adjustFromRacer)
					{
						m_fCurrentSpeed -= 0.2f;
					}
				}
				if (side < 0.0f)
				{
					//Target ship is in front of player
					if (adjustFromRacer)
					{
						m_fCurrentSpeed += 0.2f;
					}
				}
			}
		}
	}

	if(m_bBoosting)
	{
		if(m_fBoostTimer <= 0.0f)
		{
			m_fCurrentBoost = 1.0f;
			m_bBoosting = false;
			m_bRailBoosting = false;

			if (!m_bOnRail)
			{
				m_fCurrentSpeed = m_fMaxSpeed;
			}
			else
			{
				m_fCurrentSpeed = m_fRailSpeed;
			}
		}//end if

		m_fBoostTimer -= m_fTick;
	}//end if

	// Race won
	if(m_nCurrentLap == GameManager().MaxLaps() + 1)
	{
		//m_nCurrentLap = 3;
		this->m_bFinished = true;
		m_nFinishRank = m_nRacePosition;
		AccelerateTo(0.2f);
	}

	/*if(!m_bInput)
	{*/
	if (m_bIsAccelerating && m_fCurrentSpeed >  (m_fIntendedSpeed - 0.011f) && m_fCurrentSpeed < (m_fIntendedSpeed + 0.011f))
	{
		m_fCurrentSpeed = m_fIntendedSpeed;
		m_bIsAccelerating = false;
	}

	float acceleration;

	if (!m_bStartTimeUp)
	{
		acceleration = 0.5f;
	}
	else
	{
		acceleration = m_fAcceleration;
	}

	if (m_bIsAccelerating)
	{
		if (m_fCurrentSpeed < m_fIntendedSpeed)
		{
			
			m_fCurrentSpeed += acceleration * fTick;

			/*if (m_fCurrentSpeed > m_fMaxSpeed)
			{
				m_fCurrentSpeed = m_fMaxSpeed;
			}*/
		}
		else if (m_fCurrentSpeed > m_fIntendedSpeed)
		{
			m_fCurrentSpeed -= acceleration * fTick;

			if ( m_fCurrentSpeed < 0.0f)
			{
				m_fCurrentSpeed = 0.0f;
			}
		}
	}

	if (m_bOnRail && !m_bRailBoosting)
	{
		if (m_fCurrentSpeed < m_fRailSpeed)
		{
			m_fCurrentSpeed += m_fTick;

			if (m_fCurrentSpeed > m_fRailSpeed)
			{
				m_fCurrentSpeed = m_fRailSpeed;
			}
		}
	}
	else if(!m_bOnRail && !m_bBoosting)
	{
		if (m_fCurrentSpeed > m_fMaxSpeed)
		{
			m_fCurrentSpeed = m_fCurrentSpeed - (m_fTick);
		}
	}

	if (m_bPositionUpdate)
	{
		UpdatePosition(fTick);
	}

	UpdateBoundingVolume();
}

void CAIPlayer::RenderMe()
{
	BSMatrix mat = m_mTransform;
	m_RenderMat = m_mRenderTransform.matrix_dx;

	D3DMATERIAL9	 matGreen;
	Direct3D().SetMaterialGreen(matGreen);


	if (m_bDisplayWhiskers)
	{
		for (int i = 0; i < MAX_WHISKERS; ++i)
		{			
			Direct3D().GetDirect3DDevice()->SetTransform(D3DTS_WORLD, &m_vWhiskers[i].mTransform.matrix_dx);
			Direct3D().GetDirect3DDevice()->SetMaterial(&matGreen);

			m_pMeshVector->DrawSubset(0);

		} // end for

	} // end if

	mat.RotateY(D3DX_PI);
	m_RenderMat = mat.matrix_dx;
	//m_mRenderTransform = m_mTransform;
	m_mRenderTransform.axis_pos = m_mTransform.axis_pos;
 
	//BSMatrix view, viewProj;
	//view.matrix_dx = CameraManager().GetActiveCamera().GetViewMatrix();
	//viewProj.matrix_dx = m_RenderMat * view.matrix_dx * CameraManager().GetActiveCamera().GetProjectionMatrix();
	//D3DXMATRIX InvRenderMat;
	//D3DXMatrixInverse(&InvRenderMat, 0, &m_mTransform.matrix_dx);

	//BSMatrix view;
	//view.matrix_dx = CameraManager().GetActiveCamera().GetViewMatrix();
	//D3DXMATRIX viewProj = view.matrix_dx * CameraManager().GetActiveCamera().GetProjectionMatrix();

	//D3DXMATRIX matInv;
	//D3DXMatrixInverse(&matInv,0,&m_mTransform.matrix_dx);				



	//ShaderManager().Shader()->SetMatrix(m_hMatWorldViewProj, &viewProj.matrix_dx);
	//ShaderManager().Shader()->SetMatrix(m_hMatWorldIT, &InvRenderMat);
	//ShaderManager().Shader()->SetMatrix(m_hMatWorld, &m_mTransform.matrix_dx);
	//ShaderManager().Shader()->SetVector(m_hViewPosition, &(D3DXVECTOR4)view.axis_pos);
	//ShaderManager().Shader()->SetTexture(m_hTexTexture, AssetManager().GetModel(m_nModelID)->Model.texArray[0]);

	//ShaderManager().Shader()->SetMatrix(m_hMatWorldViewProj, &viewProj.matrix_dx);
	//ShaderManager().Shader()->SetMatrix(m_hMatWorldIT, &InvRenderMat);
	//ShaderManager().Shader()->SetMatrix(m_hMatWorld, &m_RenderMat);
	//ShaderManager().Shader()->SetVector(m_hViewPosition, &(D3DXVECTOR4)view.axis_pos);
	//ShaderManager().Shader()->SetTexture(m_hTexTexture, AssetManager().GetModel(m_nModelID)->Model.texArray[0]);

	//unsigned int nPasses = 0;
	//ShaderManager().Shader()->SetTechnique( ShaderManager().Shader()->GetTechniqueByName("TSM2" ) );

	//ShaderManager().Shader()->CommitChanges();

	//ShaderManager().Shader()->Begin(&nPasses,0);


	//for (int i = 0; i < (int)nPasses; i++)
	//{

		//ShaderManager().Shader()->BeginPass(i);

		Direct3D().DrawModel(AssetManager().GetModel(m_nModelID)->Model,
			//						 D3DXVECTOR3(0.002f, 0.002f, 0.002f),
			//						 D3DXVECTOR3(0.003f, 0.003f, 0.003f),
			D3DXVECTOR3(2.5f, 2.5f, 2.5f),
			//						 D3DXVECTOR3(0.5f, 0.5f, 0.5f),
			m_mRenderTransform.matrix_dx);

	//	ShaderManager().Shader()->EndPass();
	//}


	//ShaderManager().Shader()->End();
	Direct3D().GetDirect3DDevice()->SetTexture(0, NULL);

}

void CAIPlayer::ProcessCollision()
{

}

void CAIPlayer::TurnLeft(float fScale)
{
	m_fTurnAccumulator -= m_fTurnIncrement * fScale;

	if (m_fTurnAccumulator < -m_fTurnMax)
		m_fTurnAccumulator = -m_fTurnMax;
}

void CAIPlayer::TurnRight(float fScale)
{
	m_fTurnAccumulator += m_fTurnIncrement * fScale;

	if (m_fTurnAccumulator > m_fTurnMax)
		m_fTurnAccumulator = m_fTurnMax;
}

float CAIPlayer::DistanceFromPlayer()
{
	float positionDifference = this->TrackDistance() - GameManager().Player().TrackDistance();
	return positionDifference;
}

float CAIPlayer::DistanceFromWaypoint()
{
	D3DXVECTOR3 vectorFrom = this->m_mTransform.axis_pos - GameManager().Track().CourseLines()[m_nWaypoint + 1].mTransform.axis_pos;
	return D3DXVec3Length(&vectorFrom);
}


void CAIPlayer::AvoidWalls()
{
	BSMatrix	mat;
	float		fRadius = m_bvSphere.GetRadius();

	for (int i = 0; i < MAX_WHISKERS; ++i)
	{
		m_vWhiskers[i].mTransform = m_mTransform;
		m_vWhiskers[i].vStart = m_mTransform.axis_pos;

		switch (i)
		{
		case FRONT_WHISKER:
			m_vWhiskers[i].vEnd = m_mTransform.axis_pos + m_vWhiskers[i].mTransform.axis_z * (100.0f * fRadius);
			break;

		case LEFT_FRONT_WHISKER:
			m_vWhiskers[i].mTransform.RotateY(-D3DX_PI * 0.25f);
			m_vWhiskers[i].vEnd = m_mTransform.axis_pos + m_vWhiskers[i].mTransform.axis_z * (10.0f * fRadius);
			break;

		case RIGHT_FRONT_WHISKER:
			m_vWhiskers[i].mTransform.RotateY(D3DX_PI * 0.25f);
			m_vWhiskers[i].vEnd = m_mTransform.axis_pos + m_vWhiskers[i].mTransform.axis_z * (10.0f * fRadius);
			break;

			//case LEFT_WHISKER:
			//	m_vWhiskers[i].mTransform.RotateY(-D3DX_PI * 0.5f);
			//	m_vWhiskers[i].vEnd = m_mTransform.axis_pos + m_vWhiskers[i].mTransform.axis_z * (3.0f * fRadius);
			//	break;
			//
			//case RIGHT_WHISKER:
			//	m_vWhiskers[i].mTransform.RotateY(D3DX_PI * 0.5f);
			//	m_vWhiskers[i].vEnd = m_mTransform.axis_pos + m_vWhiskers[i].mTransform.axis_z * (3.0f * fRadius);
			//	break;		

		} // end switch

		D3DXVECTOR3 vec = m_vWhiskers[i].vEnd - m_vWhiskers[i].vStart;
		D3DXVECTOR3 norm;

		D3DXVec3Normalize(&norm, &vec);

		m_vWhiskers[i].fLength = D3DXVec3Length(&vec);

		m_vWhiskers[i].mTransform.Normalize();
		m_vWhiskers[i].mTransform.axis_pos += vec - norm * fRadius * 0.5f;
	} // end for

	vector<tTriangle>	pTris;
	unsigned int		uiTriCount;	
	unsigned int 		uiTriIdx;
	D3DXVECTOR3	 		vCollPoint;

	vector<CSceneObject*> mySceneObjects;
	mySceneObjects = SceneManager().GetSceneObjects();

	//CSceneObject* wall1 = mySceneObjects[2];
	//CSceneObject* wall2 = mySceneObjects[5];

	for (int i = 0; i < (int)mySceneObjects.size(); i++)
	{	// Test Left Whisker

		if (mySceneObjects[i]->GetType() == OBJECT_TRACK || mySceneObjects[i]->GetType() == OBJECT_WALL)
		{
			CollisionManager().GetTriangles(mySceneObjects[i]->Tree(), pTris, &uiTriCount, m_vWhiskers[1].vStart, m_vWhiskers[1].vEnd);

			//uiTriCount = pTris.size();

			if (pTris.size() > 0)
			{
				if (CollisionManager().LineSegment2Triangle(vCollPoint, uiTriIdx, pTris, uiTriCount, m_vWhiskers[1].vStart, m_vWhiskers[1].vEnd))
				{
					m_bSteerRight = true;
					TurnRight(1.3f);
				} // end if

			} // end if
		}

		if (mySceneObjects[i]->GetType() == OBJECT_TRACK || mySceneObjects[i]->GetType() == OBJECT_WALL)
		{
			CollisionManager().GetTriangles(mySceneObjects[i]->Tree(), pTris, &uiTriCount, m_vWhiskers[2].vStart, m_vWhiskers[2].vEnd);

			//uiTriCount = pTris.size();

			if (pTris.size() > 0)
			{
				if (CollisionManager().LineSegment2Triangle(vCollPoint, uiTriIdx, pTris, uiTriCount, m_vWhiskers[2].vStart, m_vWhiskers[2].vEnd))
				{
					m_bSteerLeft = true;
					TurnLeft(1.3f);
				} // end if

			} // end if
		}
	}

}

void CAIPlayer::Shutdown()
{
	CRacer::Shutdown();

	DispatchManager().RemoveFromBucket(UPDATE_OBJECT_BUCKET, (CBucketItem *)this);			
	DispatchManager().RemoveFromBucket(UPDATE_COLLISION_BUCKET, (CBucketItem *)this);
	DispatchManager().RemoveFromBucket(RENDER_PLAYER1_FRONTBACK_BUCKET, (CBucketItem *)this);

	if(m_nLeftExhaustID != -1)
		EffectsManager().GetEffect(m_nLeftExhaustID)->Shutdown();

	if(m_nRightExhaustID != -1)
		EffectsManager().GetEffect(m_nRightExhaustID)->Shutdown();

	if (m_Camera)
	{
		CameraManager().UnloadCamera(m_szCameraName);

		delete m_Camera;

		m_Camera = NULL;

	} 

}

