#ifndef _CAIPLAYER_H_

#define _CAIPLAYER_H_

#include "Constants.h"
#include "CDirect3D.h"
#include "CAssetManager.h"
#include "CCollisionManager.h"
#include "CBucketItem.h"
#include "CRacer.h"
	
class CShip;
class CFollowCamera;

//struct tAIWhisker
//{
//	BSMatrix	mTransform;
//	D3DXVECTOR3 vStart;
//	D3DXVECTOR3 vEnd;
//	float		fLength;
//};
	
class CAIPlayer : public CRacer
{
		bool		m_bAIActive;
		int			m_AIID;

		//vector<tAIWhisker>	m_vWhiskers;

		ID3DXMesh			*m_pMeshVector;

		int NextValidAIID(){static int NextAIID = 0; return NextAIID++;}

		CFollowCamera*	m_Camera;
		int				m_iCameraIndex;
		string			m_szCameraName;

		float				 m_fTurnAccumulator;
		float				 m_fTurnIncrement;
		float				 m_fTurnMax;

		float			m_fIntendedSpeed;
		bool			m_bIsAccelerating;

		bool			m_bBumping;
		bool			m_bSteering;
		bool			m_bRubberBand;
		bool			m_bInput;
		bool			m_bTurning;
		bool			m_bPathFollow;
		bool			m_bPositionUpdate;
		bool			m_bAI;
		bool			m_bDisplayWhiskers;
		D3DXVECTOR3		SteeringForce;

		bool			m_bSteerRight;
		bool			m_bSteerLeft;
		bool			m_bSpeedUp;
		bool			m_bSpeedDown;

		bool			m_bAlignRight;
		bool			m_bAlignLeft;

		float			m_fAlignmentToWayPoint;
		float			m_fAlignmentRange;

		float			m_fRankWeight;
		float			m_fAggressionWeight;

		float			m_fCurrentTick;
		float			m_fTick;

		int m_nLeftExhaustID;
		int m_nRightExhaustID;

		bool			m_bUpdateStartTime;
		bool			m_bStartTimeUp;
		float			m_fTimeFromStart;
		float			m_fCollisionTimer;
		float			m_fBumpTimer;
		float			m_fUseBoostTimer;

		bool			m_bAllowCatchUpSpeed;
		bool			m_bAdjustToWaypoint;
		bool			m_bAdjustingLeft;
		bool			m_bAdjustingRight;

		// AI Values

		float			m_fTimeToSpeedUp;
		float			m_fCatchUpDistance;
		float			m_fCatchUpSpeedMultiplier;
		float			m_fUseBoostDistance;
		float			m_fCatchUpOffDistance;
		float			m_fFireDistance;
		float			m_fFireWeaponTimer;
		float			m_fWeaponRechargeTime;
		int				m_nFollowWaypoint;

		float			m_fTimeOnWaypoint;
		float			m_fTimeOnWaypointSinceLastAlign;

		BSMatrix			 m_mRenderTransform;

		void InitMe();	
		void UpdateMe();	
		void RenderMe();
		void ProcessCollision();

	public:
	
		CAIPlayer();
		virtual ~CAIPlayer();
		
		void Init();
		void Update(BUCKET_ID nParentID);
		void Shutdown();

		void UpdatePathFollow();
		void UpdatePosition(float fTick);
		void UpdateInput(float fTick);
		void UpdateTerrainFollow(float fTick);
		void UpdateBoundingVolume();


		bool  ProcessCollision(CGameObject* _OBJECT);

		void TurnLeft(float fScale);
		void TurnRight(float fScale);

//		BV_TYPE		 CollisionType()	{ return BV_SPHERE; }
		CSphere		&Sphere()			{ return m_bvSphere; }
		int			 Waypoint()			{ return m_nWaypoint; }
		BSMatrix	&Transform()		{ return m_mTransform; }
		bool		IsActive()			{ return m_bAIActive; }
		float		Accumulator()			{ return m_fTurnAccumulator;}
		float		TurnIncrement()			{ return m_fTurnIncrement;}
		float		IntendedSpeed()			{ return m_fIntendedSpeed;}

		D3DXVECTOR3 GetPositon() {return m_mTransform.axis_pos;}

		void SetTransform(const BSMatrix& trasform) {m_mTransform = trasform;}

		void AlignToSpeed(CRacer* target);
		void AlignToPosition(CRacer* target);
		void AlignToPosition(BSMatrix targetTransform);

		void AccelerateTo(float speed) 
		{ 
			if(m_bAllowCatchUpSpeed || !m_bStartTimeUp)
			{
				m_fIntendedSpeed = speed;
				m_bIsAccelerating = true;
			}
			else if(speed >= 0.0f && speed <= m_fMaxSpeed)
			{
				m_fIntendedSpeed = speed;
				m_bIsAccelerating = true;
			}
		}

		float DistanceFromPlayer();

		float DistanceFromWaypoint();
		float AlignmentToWayPoint() {return m_fAlignmentToWayPoint;}
		float AlignmentRange()		{return m_fAlignmentRange;}

		void AvoidWalls();

		bool IsBumpingOn()			{ return m_bBumping;}
		bool IsSteeringOn()			{ return m_bSteering;}
		bool IsRubberBandOn()		{ return m_bRubberBand;} 
		bool IsInputOn()			{ return m_bInput;}
		bool IsTurningOn()			{ return m_bTurning;}
		bool IsPathFollowOn()		{ return m_bPathFollow;}
		bool IsPosUpdateOn()		{ return m_bPositionUpdate;}
		bool IsAIOn()				{ return m_bAI;}

		bool IsTurningRight()		{ return m_bSteerRight;}
		bool IsTurningLeft()		{ return m_bSteerLeft;}

		bool IsAligningRight()		{ return m_bAlignRight;}
		bool IsAligningLeft()		{ return m_bAlignLeft;}

		void TurnOnRubberBand() { m_bRubberBand = true; }
		void TurnOffRubberBand() { m_bRubberBand = false; }
		void TurnOnInput() { m_bInput = true; }
		void TurnOffInput() { m_bInput = false; }
		void TurnOnTurning() { m_bTurning = true;}
		void TurnOffTurning() { m_bTurning = false; }
		void TurnOnPathFollow() { m_bPathFollow = true; }
		void TurnOffPathFollow() { m_bPathFollow = false; }
		void TurnOnPositionUpdate() { m_bPositionUpdate = true; }
		void TurnOffPositionUpdate() { m_bPositionUpdate = false; }
		void TurnOnAI() { m_bAI = true; }
		void TurnOffAI() { m_bAI = false; }
		void TurnOnWhiskers() { m_bDisplayWhiskers = false; }
		void TurnOffWhiskers() {	m_bDisplayWhiskers = false; }

		void UpdateStartTime(bool b) {m_bUpdateStartTime = b;}
		void LoadAIValues();

		void SetTimeToSpeedUp(float b) {m_fTimeToSpeedUp = b;}
		void SetCatchUpDistance(float b) {m_fCatchUpDistance = b;}
		void SetCatchUpSpeedMultiplier(float b) {m_fCatchUpSpeedMultiplier = b;}
		void SetUseBoostDistance(float b) {m_fUseBoostDistance = b;}
		void SetCatchUpOffDistance(float b) {m_fCatchUpOffDistance = b;}
		void SetFollowWaypoint(int b) {m_nFollowWaypoint = b;}
		void SetWeaponRecharge(float b)	{m_fWeaponRechargeTime = b;}
		void SetFireDistance(float b) {m_fFireDistance = b;}

		void SetID(int id) {m_AIID = id;}
};
	
#endif

//void CAIPlayer::AlignToPosition(CRacer* target)
//{
//	float Distance;
//	D3DXVECTOR3 toShip;
//	D3DXVECTOR3 shipHeading;
//	D3DXVECTOR3 targetPos;
//	float side;
//
//	toShip = m_mTransform.axis_pos - target->Pos();
//	D3DXVec3Normalize(&toShip, &toShip);
//	side = D3DXVec3Dot(&toShip, &m_mTransform.axis_x);
//	Distance = D3DXVec3Length(&toShip);
//
//	if (side > 0)
//	{
//		// ON RIGHT
//		m_bAlignRight = true;
//		//TurnRight(1.0f * m_fCurrentTick);
//	}
//	else if (side < 0)
//	{
//		// ON LEFT
//		m_bAlignLeft = true;
//		//TurnLeft(1.0f * m_fCurrentTick);
//	}
//}
//
//void CAIPlayer::AlignToPosition(BSMatrix targetTransform)
//{
//	float Distance;
//	D3DXVECTOR3 toShip;
//	D3DXVECTOR3 shipHeading;
//	D3DXVECTOR3 targetPos;
//	//float side;
//
//	toShip = m_mTransform.axis_pos - targetTransform.axis_pos;
//	D3DXVec3Normalize(&toShip, &toShip);
//	m_fAlignmentToWayPoint = D3DXVec3Dot(&toShip, &m_mTransform.axis_x);
//	Distance = D3DXVec3Length(&toShip);
//	float test = (m_fHandling * m_fAlignmentToWayPoint) * 0.3f;
//
//	if (m_fTurnAccumulator < -0.02f)
//	{
//		m_bAlignLeft = false;
//		m_bAlignRight = false;
//
//		TurnLeft(1.0f);
//
//		if (m_fTurnAccumulator > 0.0f)
//		{
//			m_fTurnAccumulator = 0.0f;
//		}
//
//	} // end if
//	else if (m_fTurnAccumulator > 0.02f)
//	{
//		m_bAlignRight = false;
//		m_bAlignLeft = false;
//
//		TurnRight(1.0f);
//
//		if (m_fTurnAccumulator < 0.0f)
//		{
//			m_fTurnAccumulator = 0.0f;
//		}
//
//	} // end else if
//	else
//	{
//		m_fTurnAccumulator = 0.0f;
//	}
//
//	//if (m_fAlignmentToWayPoint > m_fAlignmentRange && !m_bAlignRight)
//	//{
//	//	// ON RIGHT
//	//	m_fTurnAccumulator = 0.0f;
//	//	TurnLeft(1.0f);
//	//}
//	//else if (m_fAlignmentToWayPoint < -m_fAlignmentRange && !m_bAlignLeft)
//	//{
//	//	// ON LEFT
//	//	m_fTurnAccumulator = 0.0f;
//	//	TurnRight(1.0f);
//	//}
//	/*else
//	{
//
//	}*/
//
//	AvoidWalls();
//}