#include "CBasicCamera.h"
#include "CCameraManager.h"
#include "CGameObject.h"

CBasicCamera::CBasicCamera()
{
	m_bIsActive = false;
	m_szName = "NONAME";
	m_fMaxPitch = D3DXToRadian( 89.0f );
	m_fMaxVelocity = 1.0f;
	m_bInvertY = FALSE;
	m_bAllowYMovement = TRUE;
	m_v3Position = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Velocity = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Offset = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3LookAt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Look = D3DXVECTOR3( 0.0f, 0.0f, 1.0f );
	m_fYaw = 0.0f;
	m_fPitch = 0.0f;
	m_fMu = 0.0f;
	m_fMuX = 0.0f;
	m_fMuY = 0.0f;
	m_fMuZ = 0.0f;
	BuildPerspective( CameraManager().DEFAULT_FOV, CameraManager().DEFAULT_ASPECT_RATIO, CameraManager().DEFAULT_NEAR_PLANE, CameraManager().DEFAULT_FAR_PLANE);
	Update();
}


CBasicCamera::CBasicCamera(const string& cameraName)
{
	m_bIsActive = false;
	m_szName = cameraName;
	m_fMaxPitch = D3DXToRadian( 89.0f );
	m_fMaxVelocity = 1.0f;
	m_bInvertY = FALSE;
	m_bAllowYMovement = TRUE;
	m_v3Position = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Velocity = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Look = D3DXVECTOR3( 0.0f, 0.0f, 1.0f );
	m_fYaw = 0.0f;
	m_fPitch = 0.0f;
	BuildPerspective( CameraManager().DEFAULT_FOV, CameraManager().DEFAULT_ASPECT_RATIO, CameraManager().DEFAULT_NEAR_PLANE, CameraManager().DEFAULT_FAR_PLANE);
	Update();
}

CBasicCamera::CBasicCamera(const string& cameraName, const D3DXVECTOR3& startPosition)
{
	m_bIsActive = false;
	m_szName = cameraName;
	m_fMaxPitch = D3DXToRadian( 89.0f );
	m_fMaxVelocity = 1.0f;
	m_bInvertY = FALSE;
	m_bAllowYMovement = TRUE;
	m_v3Position = startPosition;
	m_v3Velocity = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Look = D3DXVECTOR3( startPosition.x, startPosition.y, startPosition.z + 1.0f);
	m_fYaw = 0.0f;
	m_fPitch = 0.0f;
	BuildPerspective( CameraManager().DEFAULT_FOV, CameraManager().DEFAULT_ASPECT_RATIO, CameraManager().DEFAULT_NEAR_PLANE, CameraManager().DEFAULT_FAR_PLANE);
	Update();
}

CBasicCamera::CBasicCamera(const string& cameraName, const D3DXVECTOR3& startPosition, const D3DXVECTOR3& startLookAt)
{
	m_szName = cameraName;
	m_fMaxPitch = D3DXToRadian( 89.0f );
	m_fMaxVelocity = 1.0f;
	m_bInvertY = FALSE;
	m_bAllowYMovement = TRUE;
	m_v3Position = startPosition;
	m_v3Velocity = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Look = startLookAt;
	m_fYaw = 0.0f;
	m_fPitch = 0.0f;
	BuildPerspective( CameraManager().DEFAULT_FOV, CameraManager().DEFAULT_ASPECT_RATIO, CameraManager().DEFAULT_NEAR_PLANE, CameraManager().DEFAULT_FAR_PLANE);
	Update();
}

CBasicCamera::CBasicCamera(const string& cameraName, const D3DXVECTOR3& startPosition, CGameObject* startTarget)
{
	m_bIsActive = false;
	m_szName = cameraName;
	m_fMaxPitch = D3DXToRadian( 89.0f );
	m_fMaxVelocity = 1.0f;
	m_bInvertY = FALSE;
	m_bAllowYMovement = TRUE;
	m_v3Position = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Velocity = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_v3Look = D3DXVECTOR3( 0.0f, 0.0f, 1.0f );

	m_TargetObject = startTarget;

	m_v3Look.x = startTarget->Transform().axis_pos.x;
	m_v3Look.y = startTarget->Transform().axis_pos.y;
	m_v3Look.z = startTarget->Transform().axis_pos.z;
	D3DXVec3Normalize(&m_v3Look, &(m_v3Look - m_v3Position));

	m_fYaw = 0.0f;
	m_fPitch = 0.0f;

	BuildPerspective( CameraManager().DEFAULT_FOV, CameraManager().DEFAULT_ASPECT_RATIO, CameraManager().DEFAULT_NEAR_PLANE, CameraManager().DEFAULT_FAR_PLANE);
	Update();
}

CBasicCamera::~CBasicCamera()
{
	m_szName.clear();
}

void CBasicCamera::BuildPerspective(float fieldOfView, float aspectRatio, float nearPlane, float farPlane)
{
	m_fFOV = fieldOfView;
	m_fAspectRatio = aspectRatio;
	m_fNearPlane = nearPlane;
	m_fFarPlane = farPlane;
	D3DXMatrixPerspectiveFovLH(&m_mProjectionMatrix.matrix_dx, m_fFOV, m_fAspectRatio, m_fNearPlane, m_fFarPlane);
}

void CBasicCamera::Update(void)
{
	if (m_bIsActive)
	{
		if (D3DXVec3Length(&m_v3Velocity) > m_fMaxVelocity)
		{
			m_v3Velocity = *(D3DXVec3Normalize(&m_v3Velocity, &m_v3Velocity)) * m_fMaxVelocity;
		}

		// Move the Camera by its velocity
		m_v3Position += m_v3Velocity;

		// Could Decelerate
		m_v3Velocity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
		m_v3Look = m_v3Position + m_v3Look;

		// Calculate new view
		D3DXVECTOR3 up = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
		D3DXMatrixLookAtLH(&m_mViewMatrix.matrix_dx, &m_v3Position, &m_v3Look, & up);

		m_v3Right.x = m_mViewMatrix.matrix_dx._11;
		m_v3Right.y = m_mViewMatrix.matrix_dx._21;
		m_v3Right.z = m_mViewMatrix.matrix_dx._31;
		m_v3Up.x = m_mViewMatrix.matrix_dx._12;
		m_v3Up.y = m_mViewMatrix.matrix_dx._22;
		m_v3Up.z = m_mViewMatrix.matrix_dx._32;
		m_v3Look.x = m_mViewMatrix.matrix_dx._13;
		m_v3Look.y = m_mViewMatrix.matrix_dx._23;
		m_v3Look.z = m_mViewMatrix.matrix_dx._33;

		float lookLengthOnXZ = sqrtf(m_v3Look.z * m_v3Look.z + m_v3Look.x *m_v3Look.x);
		m_fPitch = atan2f(m_v3Look.y, lookLengthOnXZ);
		m_fYaw = atan2f(m_v3Look.x, m_v3Look.z);
	}
}

void CBasicCamera::Dolly(float distance)
{
	if ( m_bAllowYMovement )
	{
		m_v3Velocity += m_v3Look * distance;

	}
	else
	{
		D3DXVECTOR3 moveVector( m_v3Look.x, 0.0f, m_v3Look.z );
		D3DXVec3Normalize( &moveVector, &moveVector );
		moveVector *= distance;
		m_v3Velocity += moveVector;
	}
}

void CBasicCamera::Strafe(float distance)
{
	m_v3Velocity += m_v3Right * distance;
}

void CBasicCamera::MoveUp(float distance)
{
	if(m_bAllowYMovement)
	{
		m_v3Velocity.y += distance;
	}
}

void CBasicCamera::Yaw(float radians)
{
	if(radians == 0.0f)
		return;

	D3DXMATRIX rotation;

	D3DXMatrixRotationAxis(&rotation, &m_v3Up, radians);
	D3DXVec3TransformNormal(&m_v3Right, &m_v3Right, &rotation); 
	D3DXVec3TransformNormal(&m_v3Look, &m_v3Look, &rotation);
}

void CBasicCamera::Pitch(float radians)
{
	if(radians == 0.0f)
		return;

	radians = (m_bInvertY) ? -radians : radians;

	m_fPitch -= radians;

	if(m_fPitch > m_fMaxPitch)
	{
		radians += m_fPitch - m_fMaxPitch;
	}
	else if (m_fPitch < -m_fMaxPitch)
	{
		radians += m_fPitch + m_fMaxPitch;
	}

	D3DXMATRIX rotation;

	D3DXMatrixRotationAxis(&rotation, &m_v3Right, radians);
	D3DXVec3TransformNormal(&m_v3Up, &m_v3Up, &rotation); 
	D3DXVec3TransformNormal(&m_v3Look, &m_v3Look, &rotation);
}

void CBasicCamera::Roll(float radians)
{
	D3DXMATRIX rotation;

	D3DXMatrixRotationAxis(&rotation, &m_v3Look, radians);
	D3DXVec3TransformNormal(&m_v3Right, &m_v3Right, &rotation); 
	D3DXVec3TransformNormal(&m_v3Up, &m_v3Up, &rotation);
}

void CBasicCamera::SetPosition(D3DXVECTOR3* position)
{
	m_v3Position.x = position->x;
	m_v3Position.y = position->y;
	m_v3Position.z = position->z;
}

void CBasicCamera::SetLookAtTarget(D3DXVECTOR3* lookAt)
{
	m_v3Look.x = lookAt->x;
	m_v3Look.y = lookAt->y;
	m_v3Look.z = lookAt->z;
	D3DXVec3Normalize(&m_v3Look, &(m_v3Look - m_v3Position));
}
