#include "CCameraManager.h"
#include "CCamera.h"
#include "CMovableCamera.h"
#include "CHardAttachCamera.h"
#include "CGameManager.h"
#include "CInputManager.h"
#include "CScriptManager.h"
#include "CSoftAttachCamera.h"
#include "CDollyCamera.h"

CCameraManager::CCameraManager()
{
	m_szCurrentCamera = "";
	m_bIsActive = false;
	m_iCurrentCamera = 0;
}

CCameraManager::~CCameraManager()
{

}

CCameraManager &CCameraManager::GetInstance()
{
	static CCameraManager instance;

	return instance;
}

int TranslateType(string type)
{
	int ret = -1;

	if(type == "Basic")
		ret = BASIC_CAMERA;
	else if(type == "HardAttach")
		ret = HARDATTACH_CAMERA;
	else if(type == "Follow")
		ret = FOLLOW_CAMERA;
	else if(type == "Moveable")
		ret = MOVABLE_CAMERA;
	else if(type == "Dolly")
		ret = DOLLY_CAMERA;
	else if(type == "SoftAttach")
		ret = SOFTATTACH_CAMERA;

	return ret;
}

void CCameraManager::Init()
{
	m_bIsActive = true;

	// Create My initial cameras
	CScript* cameraScript = ScriptManager().Script("Camera Config");

	DEFAULT_FOV = cameraScript->ValueAsFloat("DEFAULT_FOV");
	DEFAULT_ASPECT_RATIO = cameraScript->ValueAsFloat("DEFAULT_ASPECT_RATIO");
	DEFAULT_NEAR_PLANE = cameraScript->ValueAsFloat("DEFAULT_NEAR_PLANE");
	DEFAULT_FAR_PLANE = cameraScript->ValueAsFloat("DEFAULT_FAR_PLANE");
	
	CScript* trackScript = ScriptManager().Script("Track Cameras");

	int numCamerasToLoad = 0;
	numCamerasToLoad = trackScript->ValueAsInt("NUM_CAMSTOLOAD");
	char buffer[128] ="";


	for (int i = 0; i < numCamerasToLoad; i++)
	{
		float startX, startY, startZ;
		float targetX, targetY, targetZ;
		startX = startY = startZ = 0.0f;
		targetX = targetY = targetZ = 0.0f;

		sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_TYPE", i);
		int type = TranslateType(trackScript->Value(buffer));

		sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_NAME", i);
		string name = trackScript->Value(buffer);

		sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_USEDEFAULTCONSTRUCTOR", i);
		bool useDefaultConstructor = trackScript->ValueAsBool(buffer);

		if(!useDefaultConstructor)
		{
			sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_HASSTARTPOS", i);
			bool hasStartPos = trackScript->ValueAsBool(buffer);
			bool startPosPlayer = true;
			bool isTargetPlayer = true;

			if(hasStartPos)
			{
				sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_ISSTARTPLAYER", i);
				startPosPlayer = trackScript->ValueAsBool(buffer);

				if(!startPosPlayer)
				{
					sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_STARTPOS_X", i);
					startX = trackScript->ValueAsFloat(buffer);

					sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_STARTPOS_Y", i);
					startY = trackScript->ValueAsFloat(buffer);

					sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_STARTPOS_Z", i);
					startZ = trackScript->ValueAsFloat(buffer);
				}
			}

			sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_HASSTARTTARGET", i);
			bool hasStartTarget = trackScript->ValueAsBool(buffer);
		
			if (hasStartTarget)
			{
				sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_ISTARGETPLAYER", i);
				isTargetPlayer = trackScript->ValueAsBool(buffer);

				if(!isTargetPlayer)
				{
					sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_STARTLOOK_X", i);
					targetX = trackScript->ValueAsFloat(buffer);

					sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_STARTLOOK_Y", i);
					targetY = trackScript->ValueAsFloat(buffer);

					sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_STARTLOOK_Z", i);
					targetZ = trackScript->ValueAsFloat(buffer);
				}
			}

			CCamera* temp;

			switch(type)
			{
			case (int)BASIC_CAMERA:
				if (useDefaultConstructor)
				{
					temp = new CBasicCamera(name);
				}
				else if( hasStartPos && !hasStartTarget)
				{
					if(!startPosPlayer)
						temp = new CBasicCamera(name, D3DXVECTOR3(startX, startY, startZ));
					else
						temp = new CBasicCamera(name, GameManager().Player().Transform().axis_pos);
				}
				else if( hasStartPos && hasStartTarget)
				{
					if(!startPosPlayer && !isTargetPlayer)
						temp = new CBasicCamera(name, D3DXVECTOR3(startX, startY, startZ), D3DXVECTOR3(targetX, targetY, targetZ));
					else if(!startPosPlayer && isTargetPlayer)
						temp = new CBasicCamera(name, D3DXVECTOR3(startX, startY, startZ), &GameManager().Player());
					else if(startPosPlayer && isTargetPlayer)
						temp = new CBasicCamera(name, GameManager().Player().Transform().axis_pos, &GameManager().Player());
					else if(startPosPlayer && !isTargetPlayer)
						temp = new CBasicCamera(name, GameManager().Player().Transform().axis_pos, D3DXVECTOR3(targetX, targetY, targetZ));
				}
				break;
			case (int)HARDATTACH_CAMERA:
				if (useDefaultConstructor)
				{
					temp = new CHardAttachCamera(name);
				}
				else if( hasStartPos && !hasStartTarget)
				{
					if(!startPosPlayer)
						temp = new CHardAttachCamera(name, D3DXVECTOR3(startX, startY, startZ));
					else
						temp = new CHardAttachCamera(name, GameManager().Player().Transform().axis_pos);
				}
				else if( hasStartPos && hasStartTarget)
				{
					if(!startPosPlayer && !isTargetPlayer)
						temp = new CHardAttachCamera(name, D3DXVECTOR3(startX, startY, startZ), D3DXVECTOR3(targetX, targetY, targetZ));
					else if(!startPosPlayer && isTargetPlayer)
						temp = new CHardAttachCamera(name, D3DXVECTOR3(startX, startY, startZ), &GameManager().Player());
					else if(startPosPlayer && isTargetPlayer)
						temp = new CHardAttachCamera(name, GameManager().Player().Transform().axis_pos, &GameManager().Player());
					else if(startPosPlayer && !isTargetPlayer)
						temp = new CHardAttachCamera(name, GameManager().Player().Transform().axis_pos, D3DXVECTOR3(targetX, targetY, targetZ));
				}
				break;
			case (int)MOVABLE_CAMERA:
				if (useDefaultConstructor)
				{
					temp = new CMovableCamera(name);
				}
				else if( hasStartPos && !hasStartTarget)
				{
					if(!startPosPlayer)
						temp = new CMovableCamera(name, D3DXVECTOR3(startX, startY, startZ));
					else
						temp = new CMovableCamera(name, GameManager().Player().Transform().axis_pos);
				}
				else if( hasStartPos && hasStartTarget)
				{
					if(!startPosPlayer && !isTargetPlayer)
						temp = new CMovableCamera(name, D3DXVECTOR3(startX, startY, startZ), D3DXVECTOR3(targetX, targetY, targetZ));
					else if(!startPosPlayer && isTargetPlayer)
						temp = new CMovableCamera(name, D3DXVECTOR3(startX, startY, startZ), &GameManager().Player());
					else if(startPosPlayer && isTargetPlayer)
						temp = new CMovableCamera(name, GameManager().Player().Transform().axis_pos, &GameManager().Player());
					else if(startPosPlayer && !isTargetPlayer)
						temp = new CMovableCamera(name, GameManager().Player().Transform().axis_pos, D3DXVECTOR3(targetX, targetY, targetZ));
				}
				break;
			case (int)FOLLOW_CAMERA:
				if (useDefaultConstructor)
				{
					temp = new CFollowCamera(name);
				}
				else if( hasStartPos && !hasStartTarget)
				{
					if(!startPosPlayer)
						temp = new CFollowCamera(name, D3DXVECTOR3(startX, startY, startZ));
					else
						temp = new CFollowCamera(name, GameManager().Player().Transform().axis_pos);
				}
				else if( hasStartPos && hasStartTarget)
				{
					if(!startPosPlayer && !isTargetPlayer)
						temp = new CFollowCamera(name, D3DXVECTOR3(startX, startY, startZ), D3DXVECTOR3(targetX, targetY, targetZ));
					else if(!startPosPlayer && isTargetPlayer)
						temp = new CFollowCamera(name, D3DXVECTOR3(startX, startY, startZ), &GameManager().Player());
					else if(startPosPlayer && isTargetPlayer)
						temp = new CFollowCamera(name, GameManager().Player().Transform().axis_pos, &GameManager().Player());
					else if(startPosPlayer && !isTargetPlayer)
						temp = new CFollowCamera(name, GameManager().Player().Transform().axis_pos, D3DXVECTOR3(targetX, targetY, targetZ));
				}
				break;
			case (int)DOLLY_CAMERA:
				if (useDefaultConstructor)
				{
					temp = new CDollyCamera(name);
				}
				else if( hasStartPos && !hasStartTarget)
				{
					if(!startPosPlayer)
						temp = new CDollyCamera(name, D3DXVECTOR3(startX, startY, startZ));
					else
						temp = new CDollyCamera(name, GameManager().Player().Transform().axis_pos);
				}
				else if( hasStartPos && hasStartTarget)
				{
					if(!startPosPlayer && !isTargetPlayer)
						temp = new CDollyCamera(name, D3DXVECTOR3(startX, startY, startZ), D3DXVECTOR3(targetX, targetY, targetZ));
					else if(!startPosPlayer && isTargetPlayer)
						temp = new CDollyCamera(name, D3DXVECTOR3(startX, startY, startZ), &GameManager().Player());
					else if(startPosPlayer && isTargetPlayer)
						temp = new CDollyCamera(name, GameManager().Player().Transform().axis_pos, &GameManager().Player());
					else if(startPosPlayer && !isTargetPlayer)
						temp = new CDollyCamera(name, GameManager().Player().Transform().axis_pos, D3DXVECTOR3(targetX, targetY, targetZ));
				}
				break;
			case (int)SOFTATTACH_CAMERA:
				if (useDefaultConstructor)
				{
					temp = new CSoftAttachCamera(name);
				}
				else if( hasStartPos && !hasStartTarget)
				{
					if(!startPosPlayer)
						temp = new CSoftAttachCamera(name, D3DXVECTOR3(startX, startY, startZ));
					else
						temp = new CSoftAttachCamera(name, GameManager().Player().Transform().axis_pos);
				}
				else if( hasStartPos && hasStartTarget)
				{
					if(!startPosPlayer && !isTargetPlayer)
						temp = new CSoftAttachCamera(name, D3DXVECTOR3(startX, startY, startZ), D3DXVECTOR3(targetX, targetY, targetZ));
					else if(!startPosPlayer && isTargetPlayer)
						temp = new CSoftAttachCamera(name, D3DXVECTOR3(startX, startY, startZ), &GameManager().Player());
					else if(startPosPlayer && isTargetPlayer)
						temp = new CSoftAttachCamera(name, GameManager().Player().Transform().axis_pos, &GameManager().Player());
					else if(startPosPlayer && !isTargetPlayer)
						temp = new CSoftAttachCamera(name, GameManager().Player().Transform().axis_pos, D3DXVECTOR3(targetX, targetY, targetZ));
				}
				break;
			default:
				break;
			}

			float startYaw = 0.0f;
			float startPitch = 0.0f;

			sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_HASSTARTYAW", i);
			bool hasStartYaw = trackScript->ValueAsBool(buffer);

			sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_HASSTARTPITCH", i);
			bool hasStartPitch = trackScript->ValueAsBool(buffer);

			if (hasStartYaw)
			{
				sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_STARTYAW", i);
				startYaw = trackScript->ValueAsFloat(buffer);
				temp->SetYaw(startYaw);
			}

			if (hasStartPitch)
			{
				sprintf_s(buffer, sizeof(buffer), "CAMERA_%i_STARTPITCH", i);
				startPitch = trackScript->ValueAsFloat(buffer);
				temp->SetPitch(startPitch);
			}

			m_PreLoadedCameras.push_back(temp);
			int check = LoadCamera(temp);
		}

	}

	m_szCurrentCamera = "Track_Overlook_1";
	SetCameraActive(m_szCurrentCamera, false);

	DispatchManager().AddToBucket(UPDATE_CAMERA_BUCKET, (CBucketItem*)this);
}

void CCameraManager::Update(BUCKET_ID parentID)
{
	switch (parentID)
	{
		case UPDATE_CAMERA_BUCKET:
			UpdateMe();
			break;
	
	} // end switch
	
}

void CCameraManager::UpdateMe()
{
	if(m_bIsActive)
	{
		if (InputManager().Keyboard().GetBufferedKeyEx(DIK_V))
		{
			m_iCurrentCamera++;

			if (m_iCurrentCamera >= (int)m_CameraMap.size())
			{
				m_iCurrentCamera = 0;
			}

			map<string, CCamera*>::iterator iter = m_CameraMap.begin();
			int i = 0;

			while(i != m_iCurrentCamera && iter != m_CameraMap.end())
			{
				i++;
				iter++;
			}

			SetCameraActive((*iter).second->GetName(), false);
		}

		if (InputManager().Keyboard().GetBufferedKeyEx(DIK_C))
		{
			//SetCameraActive("Player_Follow_Camera", false);
			//int softAttachIndex = GetCameraIndex("SoftAttach_Camera_1");
			string playerCamera = GameManager().Player().GetCameraName();
			
			if (m_szCurrentCamera == "SoftAttach_Camera_1" || m_szCurrentCamera != playerCamera)
			{
				SetCameraActive(playerCamera, false);
			}
			else 
			{
				SetCameraActive("SoftAttach_Camera_1", false);
			}
		}

		map<string, CCamera*>::iterator iter;
		for(iter = m_CameraMap.begin(); iter != m_CameraMap.end(); iter++)
		{
			(*iter).second->Update();
		}
	}
}

void CCameraManager::Shutdown()
{
	for ( int i = 0; i < (int)m_PreLoadedCameras.size(); i++)
	{
		if (m_PreLoadedCameras[i])
		{
			delete m_PreLoadedCameras[i];
		}
	}

	m_PreLoadedCameras.clear();
	m_CameraMap.clear();
}

int CCameraManager::LoadCamera(CCamera* addCamera)
{
	// Camera is not in the current map
	if(m_CameraMap.find(addCamera->GetName()) == m_CameraMap.end())
	{
		// Add it in
		m_CameraMap[addCamera->GetName()] = addCamera;
		return (int)m_CameraMap.size()- 1;
	}

	return false;
}

bool CCameraManager::UnloadCamera(string cameraName)
{
	// Camera is not in the current map
	map<string, CCamera*>::iterator iter = m_CameraMap.begin();
	int i = 0;

	m_CameraMap.erase(cameraName);

	return true;
}


void CCameraManager::SetCameraActive(string cameraName, bool interpolate)
{
	if(!interpolate)
	{
		if (m_ActiveCamera)
		{
			m_ActiveCamera->SetActive(false);
		}
		m_ActiveCamera = m_CameraMap[cameraName];
		m_ActiveCamera->SetActive(true);
	}
	else
	{
		m_bIsInterpolating = true;
		m_NextCamera = m_CameraMap[cameraName];
	}

	m_szCurrentCamera = cameraName;
}

CCamera& CCameraManager::GetCamera(string cameraName)
{
		return *m_CameraMap[cameraName];
}

void CCameraManager::ActivateCamera(string cameraName)
{
	if(m_CameraMap.find(cameraName) == m_CameraMap.end())
		return;

	m_CameraMap[cameraName]->SetActive(true);
}

void CCameraManager::DeactivateCamera(string cameraName)
{
	if(m_CameraMap.find(cameraName) == m_CameraMap.end())
		return;

	m_CameraMap[cameraName]->SetActive(false);
}


float CCameraManager::LinearInterpolate(float val1,float val2, float mu)
{
	return(val1*(1-mu)+val2*mu);
}
