#ifndef _CAMERAMANAGER_H_
#define _CAMERAMANAGER_H_

#define MAX_CAMERAS 8

#include <vector>
using std::vector;
#include <map>
using std::map;
#include <string>
using std::string;

#include "Constants.h"
#include "CCamera.h"
#include "CBucketItem.h"


enum 
{
	BASIC_CAMERA,
	HARDATTACH_CAMERA, 
	FOLLOW_CAMERA,
	MOVABLE_CAMERA,
	DOLLY_CAMERA,
	SOFTATTACH_CAMERA,
	NUM_CAMERA_TYPES
};

class CCameraManager : public CBucketItem
{
protected:

	map<string, CCamera*> m_CameraMap;
	CCamera* m_ActiveCamera;
	CCamera* m_NextCamera;

	vector<CCamera*> m_PreLoadedCameras;

	bool m_bIsActive;
	bool m_bIsInterpolating;
	float m_fInterpolatingTimer;

	int m_iCurrentCamera;
	string m_szCurrentCamera;


private:

	void UpdateMe();

	CCameraManager();
	CCameraManager(const CCameraManager &);
	CCameraManager &operator=(const CCameraManager &);

public:

	float DEFAULT_FOV;
	float DEFAULT_ASPECT_RATIO;
	float DEFAULT_NEAR_PLANE;
	float DEFAULT_FAR_PLANE;

	~CCameraManager();

	static CCameraManager &GetInstance();
	
	void Init();
	void Shutdown();
	void Update(BUCKET_ID nParentID);

	CCamera* CreateCamera();
	int LoadCamera(CCamera* addCamera);
	bool UnloadCamera(string cameraIndex);

	void LoadTriggerDefinitions();
	void LoadTriggers();
	float LinearInterpolate(float val1,float val2, float mu);

	
	// Accessors 
	CCamera& GetCamera(string cameraName);
	void ActivateCamera(string cameraName);
	void DeactivateCamera(string cameraName);
	CCamera& GetActiveCamera()		{ return *m_ActiveCamera;}
	bool IsActive()					{ return m_bIsActive;}


	// Mutators 
	void SetCameraActive(string cameraName, bool interpolate);
	void SetActive(bool value)		{ m_bIsActive = value;}

};



static CCameraManager &CameraManager()
{
	return CCameraManager::GetInstance();
}

#endif