#include "CShaderManager.h"
#include "CShader.h"
#include "CPixelShader.h"
#include "CVertexShader.h"

CShaderManager::CShaderManager()
{
}

CShaderManager::~CShaderManager()
{

}

CShaderManager &CShaderManager::GetInstance()
{
	static CShaderManager instance;

	return instance;
}

void CShaderManager::Init()
{
}

void CShaderManager::Shutdown()
{
	map<string, CShader*>::iterator iter = m_ShaderMap.begin();

	while (iter != m_ShaderMap.end())
	{
		CShader* temp = iter->second;
		delete temp;
		iter++;
	}

	m_ShaderMap.clear();
}

bool CShaderManager::LoadShader(int shaderType, string shaderName, string shaderFileName, string shaderEntryPoint, string shaderVersion)
{
	if (m_ShaderMap.find(shaderName) != m_ShaderMap.end())
	{
		return false;
	}

	CShader* newShader;

	switch(shaderType)
	{
	case (int)PIXEL_SHADER:
		newShader = new CPixelShader();
		break;
	case (int)VERTEX_SHADER:
		newShader = new CVertexShader();
		break;
	}

	newShader->Create(shaderName, shaderFileName, shaderEntryPoint, shaderVersion);

	m_ShaderMap[shaderName] = newShader;

	return true;
}

bool CShaderManager::UnLoadShader(string shaderName, string shaderFileName)
{
	if (m_ShaderMap.find(shaderName) == m_ShaderMap.end())
	{
		return false;
	}

	CShader* theShader = m_ShaderMap[shaderName];

	m_ShaderMap.erase(shaderName);

	delete theShader;

	return true;
}

CShader& CShaderManager::GetShader(string shaderName)
{
	return *m_ShaderMap[shaderName];
}