#define _WIN32_DCOM

#ifndef _CSOUNDMANAGER_H_
#define _CSOUNDMANAGER_H_

#include <xact3.h>

#pragma comment(lib, "ole32.lib")
#pragma comment(lib, "advapi32.lib")

class CSoundManager
{
private:

	XACTINDEX				m_iBackgroundMusic;					//Index for streaming background music

	IXACT3Engine*			m_pXACT3Engine;						//The XACT Engine
	IXACT3WaveBank*			m_pInMemoryWaveBank;				//An XACT Wave Bank - for playing sfx
	IXACT3WaveBank*			m_pStreamingWaveBank;				//An XACT Wave Bank - for playing streaming music
	IXACT3SoundBank*		m_pSoundBank;

	IXACT3Cue*				m_pZeroLatencyRevCue;				//Cue for playing streaming music

	VOID*					m_pbInMemoryWaveBank;				//A pointer to the sfx wave bank
	VOID*					m_pbSoundBank;						//A pointer to the sound bank

	HANDLE					m_hStreamingWaveBank;				//A handle to the streaming wave bank
	
	bool					m_bInitialized;						//Is the XACT Engine Initialized?
	bool					m_bAuditionMode;					//Start in Audition mode? - debugging purposes
	bool					m_bDebugMode;						//Start in Debug mode? - debugging purposes

	bool					m_bStreamingWaveBankPrepared;		//Is the wave bank prepared to stream
	bool					m_bZeroLatencyCueStop;				
	bool					m_bSongStopped;						//Has the background song stopped - need to loop?

	float					m_fMusicVolume;						//The volume for streaming music
	float					m_fFXVolume;						

	CSoundManager();
	CSoundManager(const CSoundManager&);
	CSoundManager &operator = (const CSoundManager&);

public:

	~CSoundManager();

	static CSoundManager &GetInstance();

	HRESULT Initialize(bool AuditionMode, bool DebugMode);
	void Update();
	void ShutDown();
	void Play(PCSTR soundName);
	void PlayBuffered(PCSTR soundName);
	void PlayStreaming(PCSTR soundName);
	void PlayStreamingBuffered(PCSTR soundName);
	void PauseFX();
	void PauseMusic();
	void PauseAll();
	void UnPauseFX();
	void UnPauseMusic();
	void UnPauseAll();
	void Stop(PCSTR soundName);
	void StopStreaming(PCSTR soundName);

	void WINAPI XACTNotifyCallback( const XACT_NOTIFICATION* pNotification );

	const float GetMusicVolume() const { return m_fMusicVolume; }
	const float GetFXVolume() const { return m_fFXVolume; }
	int GetSoundIndex(PCSTR soundName);

	void SetMusicVolume(float volume);
	void SetFXVolume(float volume);
	void SetVariable(PCSTR variableName, float setValue);
	
	void IncreaseMusicVolume();
	void DecreaseMusicVolume();
	void IncreaseFXVolume();
	void DecreaseFXVolume();

	float GetFXVolume()    {return m_fFXVolume;}
	float GetMusicVolume() {return m_fMusicVolume;}
};

static CSoundManager &SoundManager()
{
	return CSoundManager::GetInstance();
}

#endif