#ifndef _CTRIGGER_H_
#define _CTRIGGER_H_

#include "Constants.h"


#include <string>
using std::string;

#include <map>
using std::map;

#include <vector>
using std::vector;

class cTrigger
{
public:
	int	 type;
	bool isActive;
	float distanceToActivate;

	virtual bool Activate() = 0;
	virtual bool Deactivate() = 0;
	virtual bool Update() = 0;
	virtual bool Shutdown() = 0;

	virtual bool ShouldActivate() = 0;
	virtual bool ShouldDeactivate() = 0;
};

class cTimerTrigger : public cTrigger
{
public:
	float timeToShutdown;
	float currentTime;

	virtual bool Activate();
	virtual bool Deactivate();
	virtual bool Update();
	virtual bool Shutdown();

	virtual bool ShouldActivate();
	virtual bool ShouldDeactivate();
};

class cDistanceTrigger : public cTrigger
{
public:
	float distanceToShutdown;

	virtual bool Activate();
	virtual bool Deactivate();
	virtual bool Update();
	virtual bool Shutdown();

	virtual bool ShouldActivate();
	virtual bool ShouldDeactivate();
};

class cCameraPositionChangeTrigger : public cDistanceTrigger
{
public:
	D3DXVECTOR3 offset;
	D3DXVECTOR3 lookAt;

	virtual bool Activate();
	virtual bool Deactivate();
	virtual bool Update();
	virtual bool Shutdown();

	virtual bool ShouldActivate();
	virtual bool ShouldDeactivate();
};

class cBoostRailTrigger : public cDistanceTrigger
{
public:
	virtual bool Activate();
	virtual bool Deactivate();
	virtual bool Update();
	virtual bool Shutdown();

	virtual bool ShouldActivate();
	virtual bool ShouldDeactivate();
};

class cHudSignTrigger : public cTimerTrigger
{
public:
	int signID;

	virtual bool Activate();
	virtual bool Deactivate();
	virtual bool Update();
	virtual bool Shutdown();

	virtual bool ShouldActivate();
	virtual bool ShouldDeactivate();
};

class cRailApproachTrigger : public cTimerTrigger
{
public:
	virtual bool Activate();
	virtual bool Deactivate();
	virtual bool Update();
	virtual bool Shutdown();

	virtual bool ShouldActivate();
	virtual bool ShouldDeactivate();
};

class cCheckpointTrigger : public cTimerTrigger
{
public:
	int id;
	bool updateCurrentSound;
	float volumeToGoBackTo;

	virtual bool Activate();
	virtual bool Deactivate();
	virtual bool Update();
	virtual bool Shutdown();

	virtual bool ShouldActivate();
	virtual bool ShouldDeactivate();
};

#endif
