#include "CTriggerManager.h"
#include "CGameManager.h"
#include "CHardAttachCamera.h"
#include "CCameraManager.h"
#include "CGameTime.h"
#include "CTrigger.h"
#include "CHUD.h"

//#include "CShader.h"
//#include "CPixelShader.h"
//#include "CVertexShader.h"
//cTrigger::cTrigger()
//{
//	isActive = false;
//	distanceToActivate = 0.0f;
//}
//
//cTrigger::~cTrigger()
//{
//
//}




CTriggerManager::CTriggerManager()
{
}

CTriggerManager::~CTriggerManager()
{

}
CTriggerManager &CTriggerManager::GetInstance()
{
	static CTriggerManager instance;

	return instance;
}

void CTriggerManager::Init()
{
	// Load in the triggers from the xml file

	LoadTriggerDefinitions();
	LoadTriggers();
}



void CTriggerManager::Update()
{
	// Go through all the triggers, update if necessary, activate if necessary
	for (int i = 0; i < m_vTriggers.size(); i++)
	{
		if(m_vTriggers[i]->isActive)
		{
			m_vTriggers[i]->Update();

			if (m_vTriggers[i]->ShouldDeactivate())
			{
				m_vTriggers[i]->Deactivate();
			}
			
		}
		else  // If not active, check if we have reached a point to activate
		{
			// Right now they are all activated by the players distance along the track
			// This can be changed to a switch statement to allow for different trigger 
			// activation types

			if (m_vTriggers[i]->ShouldActivate())
			{
				m_vTriggers[i]->Activate();
			}
		}
	}

	// Reset all triggers at the end of the track
	if(GameManager().Player().ActualTrackDistance() > GameManager().Track().TotalLength())
	{
		for (int i = 0; i < m_vTriggers.size(); i++)
		{
			if (m_vTriggers[i]->isActive)
			{
				m_vTriggers[i]->Deactivate();
			}
		}
	}
}

void CTriggerManager::Shutdown()
{
	for (int i = 0; i < m_vTriggers.size(); i++)
	{
		if (m_vTriggers[i]->isActive)
		{
			m_vTriggers[i]->Shutdown();
		}

		delete m_vTriggers[i];
	}
}

//CShader& CTriggerManager::GetShader(string shaderName)
//{
//	return *m_ShaderMap[shaderName];
//}

void CTriggerManager::LoadTriggerDefinitions()
{
	CScript* triggerDefScript = ScriptManager().Script("Trigger Definitions");
	char buffer[128] ="";

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_OFFSET_X");
	m_fDefaultOffsetX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_OFFSET_Y");
	m_fDefaultOffsetY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_OFFSET_Z");
	m_fDefaultOffsetZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_LOOKAT_X");
	m_fDefaultLookAtX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_LOOKAT_Y");
	m_fDefaultLookAtY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_LOOKAT_Z");
	m_fDefaultLookAtZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR1_OFFSET_X");
	m_fAboveCar1OffsetX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR1_OFFSET_Y");
	m_fAboveCar1OffsetY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR1_OFFSET_Z");
	m_fAboveCar1OffsetZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR1_LOOKAT_X");
	m_fAboveCar1LookAtX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR1_LOOKAT_Y");
	m_fAboveCar1LookAtY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR1_LOOKAT_Z");
	m_fAboveCar1LookAtZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR2_OFFSET_X");
	m_fAboveCar2OffsetX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR2_OFFSET_Y");
	m_fAboveCar2OffsetY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR2_OFFSET_Z");
	m_fAboveCar2OffsetZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR2_LOOKAT_X");
	m_fAboveCar2LookAtX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR2_LOOKAT_Y");
	m_fAboveCar2LookAtY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ABOVECAR2_LOOKAT_Z");
	m_fAboveCar2LookAtZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "APPROACHINGRING_OFFSET_X");
	m_fApproachingRingOffsetX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "APPROACHINGRING_OFFSET_Y");
	m_fApproachingRingOffsetY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "APPROACHINGRING_OFFSET_Z");
	m_fApproachingRingOffsetZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "APPROACHINGRING_LOOKAT_X");
	m_fApproachingRingLookAtX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "APPROACHINGRING_LOOKAT_Y");
	m_fApproachingRingLookAtY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "APPROACHINGRING_LOOKAT_Z");
	m_fApproachingRingLookAtZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ONBOOSTRING_OFFSET_X");
	m_fOnBoostRingOffsetX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ONBOOSTRING_OFFSET_Y");
	m_fOnBoostRingOffsetY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ONBOOSTRING_OFFSET_Z");
	m_fOnBoostRingOffsetZ = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ONBOOSTRING_LOOKAT_X");
	m_fOnBoostRingLookAtX = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ONBOOSTRING_LOOKAT_Y");
	m_fOnBoostRingLookAtY = triggerDefScript->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "ONBOOSTRING_LOOKAT_Z");
	m_fOnBoostRingLookAtZ = triggerDefScript->ValueAsFloat(buffer);


}

void CTriggerManager::LoadTriggers()
{
	CScript* triggerScript = ScriptManager().Script("Trigger Script");
	char buffer[128] ="";

	sprintf_s(buffer, sizeof(buffer), "NUM_TRIGGERS");
	int numTriggers = triggerScript->ValueAsInt(buffer);

	D3DXVECTOR3 vec;

	int StartWaypoint;
	int EndWaypoint;
	string CameraPositionName;
	string HudSignName;

	for (int i = 0; i < numTriggers; i++)
	{
		cTrigger* newTrigger;

		sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_TYPE", i);
		string triggerTypeString = triggerScript->Value(buffer);

		int triggerType;
		if (triggerTypeString == "TRIGGER_SET_CAMERA_POSITION" )
		{
			cCameraPositionChangeTrigger* Trigger = new cCameraPositionChangeTrigger();
			
			Trigger->isActive = false;
			Trigger->type = TRIGGER_SET_CAMERA_POSITION;
			
			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_ACTIVATION_DISTANCE", i);
			Trigger->distanceToActivate = triggerScript->ValueAsFloat(buffer);

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_DEACTIVATION_DISTANCE", i);
			Trigger->distanceToShutdown = triggerScript->ValueAsFloat(buffer);

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_CAMERA_POS_NAME", i);
			CameraPositionName = triggerScript->Value(buffer);

			if (CameraPositionName == "Default")
			{
				Trigger->offset = D3DXVECTOR3(m_fDefaultOffsetX, m_fDefaultOffsetY, m_fDefaultOffsetZ);
				Trigger->lookAt = D3DXVECTOR3(m_fDefaultLookAtX, m_fDefaultLookAtY, m_fDefaultLookAtZ);
			}
			else if(CameraPositionName == "AboveCar1")
			{
				Trigger->offset = D3DXVECTOR3(m_fAboveCar1OffsetX, m_fAboveCar1OffsetY, m_fAboveCar1OffsetZ);
				Trigger->lookAt = D3DXVECTOR3(m_fAboveCar1LookAtX, m_fAboveCar1LookAtY, m_fAboveCar1LookAtZ);
			}
			else if(CameraPositionName == "AboveCar2")
			{
				Trigger->offset = D3DXVECTOR3(m_fAboveCar2OffsetX, m_fAboveCar2OffsetY, m_fAboveCar2OffsetZ);
				Trigger->lookAt = D3DXVECTOR3(m_fAboveCar2LookAtX, m_fAboveCar2LookAtY, m_fAboveCar2LookAtZ);
			}
			else if(CameraPositionName == "ApproachingRing")
			{
				Trigger->offset = D3DXVECTOR3(m_fApproachingRingOffsetX, m_fApproachingRingOffsetY, m_fApproachingRingOffsetZ);
				Trigger->lookAt = D3DXVECTOR3(m_fApproachingRingLookAtX, m_fApproachingRingLookAtY, m_fApproachingRingLookAtZ);
			}
			else if(CameraPositionName == "OnBoostRing")
			{
				Trigger->offset = D3DXVECTOR3(m_fOnBoostRingOffsetX, m_fApproachingRingOffsetY, m_fOnBoostRingOffsetZ); 
				Trigger->lookAt = D3DXVECTOR3(m_fOnBoostRingLookAtX, m_fOnBoostRingLookAtY, m_fOnBoostRingLookAtZ);
			}
			
			newTrigger = Trigger;
		}
		else if (triggerTypeString == "TRIGGER_BOOST_RAIL" )
		{
			cBoostRailTrigger* Trigger = new cBoostRailTrigger();

			Trigger->isActive = false;
			Trigger->type = TRIGGER_BOOST_RAIL;

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_ACTIVATION_DISTANCE", i);
			Trigger->distanceToActivate = triggerScript->ValueAsFloat(buffer);

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_DEACTIVATION_DISTANCE", i);
			Trigger->distanceToShutdown = triggerScript->ValueAsFloat(buffer);
			
			newTrigger = Trigger;
		}
		else if (triggerTypeString == "TRIGGER_SWITCH_ACTIVE_CAMERA")
		{
			triggerType = TRIGGER_SWITCH_ACTIVE_CAMERA;
		}
		else if (triggerTypeString == "TRIGGER_HUD_SIGN")
		{
			cHudSignTrigger* Trigger = new cHudSignTrigger();

			Trigger->isActive = false;
			Trigger->type = TRIGGER_HUD_SIGN;

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_ACTIVATION_DISTANCE", i);
			Trigger->distanceToActivate = triggerScript->ValueAsFloat(buffer);

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_DEACTIVATION_TIME", i);
			Trigger->timeToShutdown = triggerScript->ValueAsFloat(buffer);

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_SIGN_NAME", i);
			HudSignName = triggerScript->Value(buffer);

			if(HudSignName == "UpsideDown")
			{
				Trigger->signID = LOOP_OVER;
			}
			else if(HudSignName == "SharpRight")
			{
				Trigger->signID = RIGHT_TURN;
			}
			else if(HudSignName == "Drop")
			{
				Trigger->signID = SHARP_DROP;
			}
			else if(HudSignName == "SCurve")
			{
				Trigger->signID = RIGHT_S_CURVE;
			}

			newTrigger = Trigger;
		}
		else if (triggerTypeString == "TRIGGER_RAIL_APPROACH")
		{
			cRailApproachTrigger* Trigger = new cRailApproachTrigger();

			Trigger->isActive = false;
			Trigger->type = TRIGGER_HUD_SIGN;

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_ACTIVATION_DISTANCE", i);
			Trigger->distanceToActivate = triggerScript->ValueAsFloat(buffer);

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_DEACTIVATION_TIME", i);
			Trigger->timeToShutdown = triggerScript->ValueAsFloat(buffer);

			newTrigger = Trigger;
		}
		else if (triggerTypeString == "TRIGGER_CHECKPOINT")
		{
			cCheckpointTrigger* Trigger = new cCheckpointTrigger();

			Trigger->isActive = false;
			Trigger->type = TRIGGER_HUD_SIGN;

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_ACTIVATION_DISTANCE", i);
			Trigger->distanceToActivate = triggerScript->ValueAsFloat(buffer);

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_DEACTIVATION_TIME", i);
			Trigger->timeToShutdown = triggerScript->ValueAsFloat(buffer);

			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_TRACKID", i);
			Trigger->id = triggerScript->ValueAsInt(buffer);

			newTrigger = Trigger;
		}
		else
		{
			continue;
		}

		m_vTriggers.push_back(newTrigger);

		/*CHardAttachCamera* myCamera;

		myCamera = (CHardAttachCamera*)&CameraManager().GetCamera(GameManager().Player().GetCameraName());*/

		/*switch(triggerType)
		{
		case (int)TRIGGER_SET_CAMERA_POSITION:
			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_START_WAYPOINT", i);
			StartWaypoint = triggerScript->ValueAsInt(buffer);
			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_END_WAYPOINT", i);
			EndWaypoint = triggerScript->ValueAsInt(buffer);
			sprintf_s(buffer, sizeof(buffer), "TRIGGER_%i_CAMERA_POS_NAME", i);
			CameraPositionName = triggerScript->Value(buffer);

			if (CameraPositionName == "Default")
			{
				myCamera->SetCameraPosTrigger(StartWaypoint, EndWaypoint,D3DXVECTOR3(m_fDefaultOffsetX, m_fDefaultOffsetY, m_fDefaultOffsetZ), D3DXVECTOR3(m_fDefaultLookAtX, m_fDefaultLookAtY, m_fDefaultLookAtZ));
			}
			else if(CameraPositionName == "AboveCar1")
			{
				myCamera->SetCameraPosTrigger(StartWaypoint, EndWaypoint,D3DXVECTOR3(m_fAboveCar1OffsetX, m_fAboveCar1OffsetY, m_fAboveCar1OffsetZ), D3DXVECTOR3(m_fAboveCar1LookAtX, m_fAboveCar1LookAtY, m_fAboveCar1LookAtZ));
			}
			else if(CameraPositionName == "AboveCar2")
			{
				myCamera->SetCameraPosTrigger(StartWaypoint, EndWaypoint,D3DXVECTOR3(m_fAboveCar2OffsetX, m_fAboveCar2OffsetY, m_fAboveCar2OffsetZ), D3DXVECTOR3(m_fAboveCar2LookAtX, m_fAboveCar2LookAtY, m_fAboveCar2LookAtZ));
			}
			else if(CameraPositionName == "ApproachingRing")
			{
				myCamera->SetCameraPosTrigger(StartWaypoint, EndWaypoint,D3DXVECTOR3(m_fApproachingRingOffsetX, m_fApproachingRingOffsetY, m_fApproachingRingOffsetZ), D3DXVECTOR3(m_fApproachingRingLookAtX, m_fApproachingRingLookAtY, m_fApproachingRingLookAtZ));
			}
			else if(CameraPositionName == "OnBoostRing")
			{
				myCamera->SetCameraPosTrigger(StartWaypoint, EndWaypoint,D3DXVECTOR3(m_fOnBoostRingOffsetX, m_fApproachingRingOffsetY, m_fOnBoostRingOffsetZ), D3DXVECTOR3(m_fOnBoostRingLookAtX, m_fOnBoostRingLookAtY, m_fOnBoostRingLookAtZ));
			}
			break;
		case (int)TRIGGER_SWITCH_ACTIVE_CAMERA:
			break;
		default:
			break;
		}*/
	}
}